;;; ===========================================================================
;;; ===========================================================================
;;; == CLASS:     DRIVER
;;; == FILE NAME: driver.clp
;;; ==
;;; == SPECS:     This file contains the definitions for the "driver" class.
;;; ==            The following slots (attributes) are defined for the class
;;; ==            and thus for each driver instance:
;;; ==
;;; ==            - Driver's name
;;; ==            - How good the driver is at passing others
;;; ==            - How aggressive the driver is
;;; ==            - Current lap driver is in,
;;; ==            - Time taken by driver during current lap
;;; ==            - Reaction time of driver
;;; ==
;;; ==            The following message handlers are defined:
;;; ==
;;; ==            - SetDriverLapCount: Sets the driver lap count,
;;; ==            - SetDriverCurrentLapTime: Sets the driver's current lap time,
;;; ==            - SetDriverBrakingStyle: Set the driver's braking style
;;; ==
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; ==                          CLASS DEFINITION
;;; ===========================================================================

(defclass DRIVER
        (is-a USER)
        (role concrete)

;;; ===========================================================================
;;; ==                               SLOTS
;;; ===========================================================================
;;; == The following slots will be defined at the time of declaration
;;; ===========================================================================

        (slot driver_name
                (type STRING))

        (slot driver_passing-skills
                (type INTEGER)
                (range 1 5)
                (default 3))

        (slot driver_aggressiveness
                (type INTEGER)
                (range 1 5)
                (default 3))
                
;;; ===========================================================================
;;; == The following slots will be defined during execution
;;; ===========================================================================

        (slot driver_lap-count
                (type INTEGER))

        (slot driver_current-lap-time
                (type FLOAT))


        (slot driver_breaking-style
                (type SYMBOL)
                (allowed-symbols early normal late))
)

;;; ===========================================================================
;;; ==                         MESSAGE HANDLERS
;;; ===========================================================================

(defmessage-handler DRIVER SetDriverLapCount (?tracklaps)
	(printout t "Enter " (send ?self get-driver_name) "'s lap count [1 - " ?tracklaps "]: ") 
	(bind ?arg (read))
	(if (or (not (numberp ?arg)) (or (> ?arg ?tracklaps) (< ?arg 1) ) )
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetDriverLapCount ?tracklaps)
		else
			(send ?self put-driver_lap-count ?arg)
			(printout t "")
	)
)

(defmessage-handler DRIVER SetDriverCurrentLapTime()
        (printout t "Enter " (send ?self get-driver_name) "'s current lap time [minutes]: ") 
        (bind ?arg (read))
        (if (not (numberp ?arg))
                then 
                        (printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
                        (send ?self SetDriverCurrentLapTime)
                else
                        (send ?self put-driver_current-lap-time ?arg)
                        (printout t "")
        )
)

(defmessage-handler DRIVER SetDriverBrakingStyle()
        (printout t "Enter " (send ?self get-driver_name) "'s braking style for the session [early,normal,late]: ") 
        (bind ?arg (read))
        (if (eq (member ?arg (slot-allowed-values DRIVER driver_breaking-style)) FALSE)
                then 
                        (printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
                        (send ?self SetDriverBrakingStyle)
                else
                        (send ?self put-driver_breaking-style ?arg)
                        (printout t "")
        )
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                          END OF DRIVER CLASS
;;; ===========================================================================
;;; ===========================================================================