;;; ===========================================================================
;;; ===========================================================================
;;; == CLASS:     TRACK
;;; == FILE NAME: track.clp
;;; ==
;;; == SPECS:     This file contains the definitions for the "track" class.
;;; ==            The following slots (attributes) are defined for the class
;;; ==            and thus for each track instance:
;;; ==
;;; ==            - Track name
;;; ==            - Track session
;;; ==            - Track length
;;; ==            - Number of laps in track
;;; ==            - Surface type of track
;;; ==            - Passing difficulty of track
;;; ==            - Race direction (counter/clockwise)
;;; ==
;;; ==            The following message handlers are defined:
;;; ==
;;; ==            - SetTrack: Based on track name loads the track's attributes
;;; ==            - SetSession: Sets the track session
;;; ==
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; ==                          CLASS DEFINITION
;;; ===========================================================================

(defclass TRACK
	(is-a USER)
	(role concrete)
	
;;; ===========================================================================
;;; ==                               SLOTS
;;; ===========================================================================
	(slot track_name
		(type SYMBOL)
		(allowed-symbols None Australia Malaysia Bahrain Spain Turkey Monaco Canada France UK Germany Hungary Europe Belgium Italy Singapore Japan China Brazil))
		
	(slot track_session
		(type SYMBOL)
		(allowed-symbols None Q1 Q2 Q3 Race))

	(slot track_length
		(type FLOAT))

	(slot track_laps
		(type INTEGER))

	(slot track_surface-type
		(type SYMBOL)
		(allowed-symbols Abrasive SemiAbrasive NonAbrasive))

	(slot track_passing-dificulty
		(type INTEGER)
		(range 1 5))

	(slot track_race-direction
		(type SYMBOL)
		(allowed-symbols clockwise counterclockwise))
)

;;; ===========================================================================
;;; ==                         MESSAGE HANDLERS
;;; ===========================================================================

(defmessage-handler TRACK SetTrack ()
	(printout t "Enter the track name: ") 
	(bind ?arg (read))
	(if (eq (member ?arg (slot-allowed-values TRACK track_name)) FALSE)
		then 
			(printout t "   -- Please verify the track name. " ?arg " is NOT a valid track." crlf)
			(send ?self SetTrack)
		else 
			(if (eq ?arg Australia)
				then 
					(send ?self put-track_name Australia)
					(send ?self put-track_length 5.303)
					(send ?self put-track_laps 58)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Malaysia)
				then 
					(send ?self put-track_name Malaysia)
					(send ?self put-track_length 5.543)
					(send ?self put-track_laps 56)
					(send ?self put-track_surface-type SemiAbrasive)
					(send ?self put-track_passing-dificulty 4)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Bahrain)
				then 
					(send ?self put-track_name Bahrain)
					(send ?self put-track_length 5.412)
					(send ?self put-track_laps 57)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Spain)
				then 
					(send ?self put-track_name Spain)
					(send ?self put-track_length 4.655)
					(send ?self put-track_laps 66)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 2)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Turkey)
				then 
					(send ?self put-track_name Turkey)
					(send ?self put-track_length 5.338)
					(send ?self put-track_laps 58)
					(send ?self put-track_surface-type SemiAbrasive)
					(send ?self put-track_passing-dificulty 4)
					(send ?self put-track_race-direction counterclockwise)

			)
			(if (eq ?arg Monaco)
				then 
					(send ?self put-track_name Monaco)
					(send ?self put-track_length 3.340)
					(send ?self put-track_laps 78)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 5)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Canada)
				then 
					(send ?self put-track_name Canada)
					(send ?self put-track_length 4.361)
					(send ?self put-track_laps 70)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 4)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg France)
				then 
					(send ?self put-track_name France)
					(send ?self put-track_length 4.411)
					(send ?self put-track_laps 70)
					(send ?self put-track_surface-type SemiAbrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg UK)
				then 
					(send ?self put-track_name UK)
					(send ?self put-track_length 5.141)
					(send ?self put-track_laps 60)
					(send ?self put-track_surface-type SemiAbrasive)
					(send ?self put-track_passing-dificulty 2)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Germany)
				then 
					(send ?self put-track_name Germany)
					(send ?self put-track_length 4.574)
					(send ?self put-track_laps 67)
					(send ?self put-track_surface-type Abrasive)
					(send ?self put-track_passing-dificulty 4)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Hungary)
				then 
					(send ?self put-track_name Hungary)
					(send ?self put-track_length 4.381)
					(send ?self put-track_laps 70)
					(send ?self put-track_surface-type Abrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)

			)
			(if (eq ?arg Europe)
				then 
					(send ?self put-track_name Europe)
					(send ?self put-track_length 5.440)
					(send ?self put-track_laps 57)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 5)
					(send ?self put-track_race-direction clockwise)
			
			)
			(if (eq ?arg Belgium)
				then 
					(send ?self put-track_name Belgium)
					(send ?self put-track_length 7.004)
					(send ?self put-track_laps 44)
					(send ?self put-track_surface-type Abrasive)
					(send ?self put-track_passing-dificulty 2)
					(send ?self put-track_race-direction clockwise)
			
			)
			(if (eq ?arg Italy)
				then 
					(send ?self put-track_name Italy)
					(send ?self put-track_length 5.793)
					(send ?self put-track_laps 53)
					(send ?self put-track_surface-type Abrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)
			
			)
			(if (eq ?arg Singapore)
				then 
					(send ?self put-track_name Singapore)
					(send ?self put-track_length 5.067)
					(send ?self put-track_laps 61)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 4)
					(send ?self put-track_race-direction counterclockwise)
			
			)
			(if (eq ?arg Japan)
				then 
					(send ?self put-track_name Japan)
					(send ?self put-track_length 4.563)
					(send ?self put-track_laps 67)
					(send ?self put-track_surface-type NonAbrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)
			
			)
			(if (eq ?arg China)
				then 
					(send ?self put-track_name China)
					(send ?self put-track_length 5.451)
					(send ?self put-track_laps 56)
					(send ?self put-track_surface-type SemiAbrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction clockwise)
			
			)
			(if (eq ?arg Brazil)
				then 
					(send ?self put-track_name Brazil)
					(send ?self put-track_length 4.309)
					(send ?self put-track_laps 71)
					(send ?self put-track_surface-type Abrasive)
					(send ?self put-track_passing-dificulty 3)
					(send ?self put-track_race-direction counterclockwise)
			
			)
			(printout t "")
	)
)

(defmessage-handler TRACK SetSession ()
	(printout t "Select Session (Q1/Q2/Q3/Race): ") 
	(bind ?arg (read))
	(if (eq (member ?arg (slot-allowed-values TRACK track_session)) FALSE)
		then 
			(printout t "   -- Please enter a valid session. " ?arg " is NOT a valid session." crlf)
			(send ?self SetSession)
		else
			(send ?self put-track_session ?arg)
			(printout t "")
	)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                          END OF TRACK CLASS
;;; ===========================================================================
;;; ===========================================================================